//---------------------------------------------------------------------------
//
//                      WWW.NATUREWIZARD.COM
//                      ~~~~~~~~~~~~~~~~~~~~
//
//              Author      : Dr. Heinz Seyringer
//              Project     : Tutorial 1 - Transparency
//              Description : This project demonstrates how to use transparent
//                            surfaces
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "MAIN.h"
#include "math.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMain *FormMain;
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetupLighting()
{
   GLfloat ambientLight[]={0.1,0.1,0.1,1.0};    // set ambient light parameters
   glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);

   GLfloat diffuseLight[]={0.8,0.8,0.8,1.0};    // set diffuse light parameters
   glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);

   GLfloat specularLight[]={0.5,0.5,0.5,1.0};   // set specular light parameters
   glLightfv(GL_LIGHT0,GL_SPECULAR,specularLight);

   GLfloat lightPos[]={0.0,30.0,60.0,0.0};      // set light position
   glLightfv(GL_LIGHT0,GL_POSITION,lightPos);

   GLfloat specularReflection[]={1.0,1.0,1.0,1.0};  // set specularity
   glMaterialfv(GL_FRONT, GL_SPECULAR, specularReflection);
   glMateriali(GL_FRONT,GL_SHININESS,128);

   glEnable(GL_LIGHT0);                         // activate light0
   glEnable(GL_LIGHTING);                       // enable lighting
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight); // set light model
   glEnable(GL_COLOR_MATERIAL);                 // activate material
   glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);
   glEnable(GL_NORMALIZE);                      // normalize normal vectors
}
//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
    : TForm(Owner)
{
    Application->OnIdle = IdleLoop;   // Define idle function
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::IdleLoop(TObject*, bool& done)
{
    done = false;               // request more idle time from the system
    RenderGLScene();
    SwapBuffers(hdc);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
    hdc = GetDC(Handle);           // get device context from main window
    SetPixelFormatDescriptor();
    hrc = wglCreateContext(hdc);   // use device context to create
                                   // a rendering context
    if(hrc == NULL)
    	ShowMessage("Creating Rendering Context failed.");
    wglMakeCurrent(hdc,hrc);       // tell windows to use hdc and rdc
    if(wglMakeCurrent(hdc, hrc) == false)
    	ShowMessage("MakeCurrent of rendering context failed.");
    w = ClientWidth;               // width of main windows client area
    h = ClientHeight;              // height of main windows client area

    glEnable(GL_DEPTH_TEST);

    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);  // clear background to black
    glClearDepth(100.0);                   // set depth buffer to the most distant value
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    SetupLighting();                // initialize the lighting and materials
    alpha=0.0;
    transparency = 0.0;
    transflag=1.0;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetPixelFormatDescriptor()
{
    PIXELFORMATDESCRIPTOR pfd = {
    	sizeof(PIXELFORMATDESCRIPTOR),       // size of the structure
        1,                                   // structure version
        PFD_DRAW_TO_WINDOW |                 // draw directly to window
                                             // (not to a bitmap file)
        PFD_SUPPORT_OPENGL |                 // allow DC to support opengl calls
        PFD_DOUBLEBUFFER,                    // use double buffer
        PFD_TYPE_RGBA,                       // use RGBA color mode (A = alpha)
        24,                                  // use 24 bit color
        0,0,0,0,0,0,0,0,0,0,0,0,0,           // use default bitplanes
        32,                                  // use 32 bit z buffer
        0,0,                                 // not (yet) used
        PFD_MAIN_PLANE,                      // draw to main plane
        0,0,0,                               // different mask bits
    };
    PixelFormat = ChoosePixelFormat(hdc, &pfd);  // choose and set the
    SetPixelFormat(hdc, PixelFormat, &pfd);      // appropriate pixelformat
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormResize(TObject *Sender)
{

    GLfloat Range = 100.0;  // numerical range on the 3 axes

    w = ClientWidth;      // check new client width of the main window
    h = ClientHeight;     // check new client height of the main window

    if(h == 0)            // prevent a division by zero, by making sure that
    	h = 1;            // windows height is at least 1

    glViewport(0, 0, w, h);         // reset viewport


    glMatrixMode(GL_PROJECTION);    // add perspective to scene
    glLoadIdentity();               // restore matrix to original state

    if (w <= h)
        glOrtho (-Range, Range, -Range*h/w, Range*h/w, -Range, Range);
    else
	glOrtho (-Range*w/h, Range*w/h, -Range, Range, -Range, Range);

    glMatrixMode(GL_MODELVIEW);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::RenderGLScene()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    DrawObjects();
    glFlush();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::DrawObjects()
{

   // rendering functions

  if (transparency>1.0) {
    transflag=0.0;
    transparency-=0.01;
  }
  if (transparency<0.0) {
    transflag=1.0;
    transparency+=0.01;
  }
  if (transflag==0.0) transparency-=0.01;
  if (transflag==1.0) transparency+=0.01;

  glColor3f(1.0,1.0,1.0);
  float z1,z2,z3;         // calculate the heigths of the points
  glLoadIdentity();                             // initialize drawing coordinates
  glRotatef(20.0,1.0,0.0,0.0);
  glRotatef(alpha,0.0,1.0,0.0);
  alpha+=1.0;
  glScalef(0.8,0.8,0.8);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glDisable(GL_BLEND);
  glutSolidTeapot(70.0);

  glLoadIdentity();
  glEnable(GL_BLEND);
  glColor4f(0.0,0.0,1.0,transparency);
  glBegin(GL_QUADS);
    glVertex3f( 0.0,-80.0,100.0);
    glVertex3f(80.0,-80.0,100.0);
    glVertex3f(80.0, 80.0,100.0);
    glVertex3f( 0.0, 80.0,100.0);
  glEnd();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormPaint(TObject *Sender)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    DrawObjects();
    glFlush();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
{
    wglMakeCurrent(NULL, NULL);
    wglDeleteContext(hrc);
}

